@@ -1,8 +1,9 @@
 # coding: utf-8
-# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
 # This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
 
 import platform
+import oci.util
 from .version import __version__
 from oci._vendor.requests.exceptions import RequestException as BaseRequestException
 from oci._vendor.requests.exceptions import ConnectTimeout as BaseConnectTimeout
@@ -28,7 +29,7 @@ class ServiceError(Exception):
         self.client_version = kwargs.get('client_version')
         self.timestamp = kwargs.get('timestamp')
 
-        api_errors_info = "See https://docs.oracle.com/iaas/Content/API/References/apierrors.htm#apierrors_{}__{}_{} for more information about resolving this error.".format(str(self.status), str(self.status), str(self.code).lower())
+        api_errors_info = "See https://docs.oracle.com/iaas/Content/API/References/apierrors.htm#apierrors_{}__{}_{} for more information about resolving this error.".format(str(self.status), str(self.status), str(code).lower())
         contact_info = "If you are unable to resolve this {} issue, please contact Oracle support and provide them this full error message.".format(self.target_service)
 
         if not message:
@@ -52,6 +53,14 @@ class ServiceError(Exception):
         else:
             error_details['troubleshooting_tips'] = "{} {}".format(api_errors_info, contact_info)
 
+        if isinstance(kwargs.get('deserialized_data'), dict):
+            # convert the Keys of the dictionary to snake case
+            deserialized_data = oci.util.camel_to_snake_keys(kwargs.get('deserialized_data'))
+            for key, value in deserialized_data.items():
+                # Skip adding duplicate keys in error_details
+                if key not in (error_details):
+                    error_details[key] = value
+
         super(ServiceError, self).__init__(error_details)
 
     def _get_opc_request_id(self):
@@ -95,6 +104,14 @@ class InvalidConfig(ClientError):
         return str(self.errors)
 
 
+class InvalidAlloyConfig(ClientError):
+    """Alloy config is invalid, or is blocking a service"""
+
+
+class InvalidResourcePrincipalArguments(ClientError):
+    """The ResourceId is missing."""
+
+
 class InvalidPrivateKey(ClientError):
     """The provided key is not a private key, or the provided passphrase is incorrect."""
 
