@@ -1,5 +1,5 @@
 # coding: utf-8
-# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
 # This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
 
 REGIONS_SHORT_NAMES = {
@@ -34,6 +34,10 @@ REGIONS_SHORT_NAMES = {
     'arn': 'eu-stockholm-1',
     'jnb': 'af-johannesburg-1',
     'cdg': 'eu-paris-1',
+    'qro': 'mx-queretaro-1',
+    'mad': 'eu-madrid-1',
+    'ord': 'us-chicago-1',
+    'mty': 'mx-monterrey-1',
     'lfi': 'us-langley-1',
     'luf': 'us-luke-1',
     'ric': 'us-gov-ashburn-1',
@@ -44,7 +48,16 @@ REGIONS_SHORT_NAMES = {
     'nja': 'ap-chiyoda-1',
     'ukb': 'ap-ibaraki-1',
     'mct': 'me-dcc-muscat-1',
-    'wga': 'ap-dcc-canberra-1'
+    'wga': 'ap-dcc-canberra-1',
+    'bgy': 'eu-dcc-milan-1',
+    'mxp': 'eu-dcc-milan-2',
+    'snn': 'eu-dcc-dublin-2',
+    'dtm': 'eu-dcc-rating-2',
+    'dus': 'eu-dcc-rating-1',
+    'ork': 'eu-dcc-dublin-1',
+    'vll': 'eu-madrid-2',
+    'str': 'eu-frankfurt-2',
+    'beg': 'eu-jovanovac-1'
 }
 REGION_REALMS = {
     'ap-chuncheon-1': 'oc1',
@@ -78,6 +91,10 @@ REGION_REALMS = {
     'eu-stockholm-1': 'oc1',
     'af-johannesburg-1': 'oc1',
     'eu-paris-1': 'oc1',
+    'mx-queretaro-1': 'oc1',
+    'eu-madrid-1': 'oc1',
+    'us-chicago-1': 'oc1',
+    'mx-monterrey-1': 'oc1',
 
     'us-langley-1': 'oc2',
     'us-luke-1': 'oc2',
@@ -94,7 +111,19 @@ REGION_REALMS = {
 
     'me-dcc-muscat-1': 'oc9',
 
-    'ap-dcc-canberra-1': 'oc10'
+    'ap-dcc-canberra-1': 'oc10',
+
+    'eu-dcc-milan-1': 'oc14',
+    'eu-dcc-milan-2': 'oc14',
+    'eu-dcc-dublin-2': 'oc14',
+    'eu-dcc-rating-2': 'oc14',
+    'eu-dcc-rating-1': 'oc14',
+    'eu-dcc-dublin-1': 'oc14',
+
+    'eu-madrid-2': 'oc19',
+    'eu-frankfurt-2': 'oc19',
+
+    'eu-jovanovac-1': 'oc20'
 }
 REALMS = {
     'oc1': 'oraclecloud.com',
@@ -103,7 +132,10 @@ REALMS = {
     'oc4': 'oraclegovcloud.uk',
     'oc8': 'oraclecloud8.com',
     'oc9': 'oraclecloud9.com',
-    'oc10': 'oraclecloud10.com'
+    'oc10': 'oraclecloud10.com',
+    'oc14': 'oraclecloud14.com',
+    'oc19': 'oraclecloud.eu',
+    'oc20': 'oraclecloud20.com'
 }
 REGIONS = [
     'ap-chuncheon-1',
@@ -137,6 +169,10 @@ REGIONS = [
     'eu-stockholm-1',
     'af-johannesburg-1',
     'eu-paris-1',
+    'mx-queretaro-1',
+    'eu-madrid-1',
+    'us-chicago-1',
+    'mx-monterrey-1',
     'us-langley-1',
     'us-luke-1',
     'us-gov-ashburn-1',
@@ -147,5 +183,14 @@ REGIONS = [
     'ap-chiyoda-1',
     'ap-ibaraki-1',
     'me-dcc-muscat-1',
-    'ap-dcc-canberra-1'
+    'ap-dcc-canberra-1',
+    'eu-dcc-milan-1',
+    'eu-dcc-milan-2',
+    'eu-dcc-dublin-2',
+    'eu-dcc-rating-2',
+    'eu-dcc-rating-1',
+    'eu-dcc-dublin-1',
+    'eu-madrid-2',
+    'eu-frankfurt-2',
+    'eu-jovanovac-1'
 ]
