@@ -1,5 +1,5 @@
 # coding: utf-8
-# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
 # This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
 
 # Contains classes for defining and building retry strategies.
@@ -329,7 +329,7 @@ class ExponentialBackoffRetryStrategyBase(object):
                             raise
 
                     if log_info["logger"] and log_info["debug"]:
-                        log_info["logger"].debug("Retry attempt: {}, Http Status Code: {}, Error Code: {}".format(attempt, status_code, error_code))
+                        log_info["logger"].debug("Retry attempt: {}, Http Status Code: {}, Error Code: {}, Exception: {}".format(attempt, status_code, error_code, e))
                 else:
                     raise
 
